@@ -132,7 +132,10 @@
 #' @param composite_function Character of length 1: The name of a
 #' function used to combine downloaded images into a single composite
 #' (i.e., to aggregate pixel values from multiple images into a single value).
-#' Must be one of of "sum", "mean", "median", "min", "max".
+#' Options include "merge", which 'stamps' images on top of one another such that
+#' the "last" value downloaded for a pixel -- which isn't guaranteed to be the most
+#' recent one -- will be the only value used, or any of "sum", "mean", "median", 
+#' "min", or "max", which consider all values available at each pixel.
 #' Set to `NULL` to not composite
 #' (i.e., to rescale and save each individual file independently).
 #' @inheritParams rstac::stac_search
@@ -140,9 +143,11 @@
 #' argument of [sf::gdal_utils()]. The same set of options are used for all
 #' downloaded data and the final output images; this means that some common
 #' options (for instance, `PREDICTOR=3`) may cause errors if bands are of
-#' varying data types.
+#' varying data types. The default values are provided by 
+#' [rsi_gdalwarp_options()].
 #' @param gdal_config_options Options passed to `gdalwarp` through the
-#' `config_options` argument of [sf::gdal_utils()].
+#' `config_options` argument of [sf::gdal_utils()]. The default values are
+#' provided by [rsi_gdal_config_options()].
 #' @param platforms The names of Landsat satellites to download imagery from.
 #' These do not correspond to the `platforms` column in [spectral_indices()];
 #' the default argument of `c("landsat-9", "landsat-8")` corresponds to
@@ -170,15 +175,73 @@
 #'   mask_band = "qa_pixel",
 #'   mask_function = landsat_mask_function,
 #'   item_filter_function = landsat_platform_filter,
-#'   platforms = c("landsat-9", "landsat-8")
+#'   platforms = c("landsat-9", "landsat-8"),
+#'   output_filename = tempfile(fileext = ".tif")
 #' )
 #'
 #' # or, mostly equivalently (will download more bands):
 #' landsat_image <- get_landsat_imagery(
 #'   aoi,
 #'   start_date = "2022-06-01",
-#'   end_date = "2022-08-30"
+#'   end_date = "2022-08-30",
+#'   output_filename = tempfile(fileext = ".tif")
 #' )
+#' 
+#' landsat_image |> 
+#'   terra::rast() |>
+#'   terra::stretch() |>
+#'   terra::plotRGB()
+#' 
+#' # The `get_*_imagery()` functions will download 
+#' # all available "data" assets by default
+#' # (usually including measured values, and excluding derived bands)
+#' sentinel1_data <- get_sentinel1_imagery(
+#'   aoi,
+#'   start_date = "2022-06-01",
+#'   end_date = "2022-07-01",
+#'   output_filename = tempfile(fileext = ".tif")
+#' )
+#' names(terra::rast(sentinel1_data))
+#' 
+#' # You can see what bands will be downloaded by a function
+#' # by inspecting the corresponding `band_mapping` object:
+#' sentinel2_band_mapping$planetary_computer_v1
+#' 
+#' # And you can add additional assets using `c()`:
+#' c(
+#'   sentinel2_band_mapping$planetary_computer_v1,
+#'   "scl"
+#' )
+#' 
+#' # Or subset the assets downloaded using `[` or `[[`:
+#' sentinel2_imagery <- get_sentinel2_imagery(
+#'   aoi,
+#'   start_date = "2022-06-01",
+#'   end_date = "2022-07-01",
+#'   asset_names = sentinel2_band_mapping$planetary_computer_v1["B01"],
+#'   output_filename = tempfile(fileext = ".tif")
+#' )
+#' names(terra::rast(sentinel2_imagery))
+#' 
+#' # If you're downloading data for a particularly large AOI,
+#' # and can't composite the resulting images or want to make
+#' # sure you can continue an interrupted download,
+#' # consider tiling your AOI and requesting each tile separately:
+#' aoi <- sf::st_make_grid(aoi, n = 2)
+#' tiles <- lapply(
+#'   seq_along(aoi),
+#'   function(i) {
+#'     get_landsat_imagery(
+#'       aoi[i],
+#'       start_date = "2022-06-01",
+#'       end_date = "2022-08-30",
+#'       output_filename = tempfile(fileext = ".tif")
+#'     )
+#'   }
+#' )
+#' # You'll get a list of tiles that you can then composite or 
+#' # work with however you wish:
+#' unlist(tiles)
 #'
 #' @export
 get_stac_data <- function(aoi,
@@ -200,26 +263,8 @@ get_stac_data <- function(aoi,
                           output_filename = paste0(proceduralnames::make_english_names(1), ".tif"),
                           composite_function = c("merge", "median", "mean", "sum", "min", "max"),
                           limit = 999,
-                          gdalwarp_options = c(
-                            "-r", "bilinear",
-                            "-multi",
-                            "-overwrite",
-                            "-co", "COMPRESS=DEFLATE",
-                            "-co", "PREDICTOR=2",
-                            "-co", "NUM_THREADS=ALL_CPUS"
-                          ),
-                          gdal_config_options = c(
-                            VSI_CACHE = "TRUE",
-                            GDAL_CACHEMAX = "30%",
-                            VSI_CACHE_SIZE = "10000000",
-                            GDAL_HTTP_MULTIPLEX = "YES",
-                            GDAL_INGESTED_BYTES_AT_OPEN = "32000",
-                            GDAL_DISABLE_READDIR_ON_OPEN = "EMPTY_DIR",
-                            GDAL_HTTP_VERSION = "2",
-                            GDAL_HTTP_MERGE_CONSECUTIVE_RANGES = "YES",
-                            GDAL_NUM_THREADS = "ALL_CPUS",
-                            GDAL_HTTP_USERAGENT = "rsi (https://permian-global-research.github.io/rsi/)"
-                          )) {
+                          gdalwarp_options = rsi_gdalwarp_options(),
+                          gdal_config_options = rsi_gdal_config_options()) {
   # query |> filter |> download |> mask |> composite |> rescale
   if (!(inherits(aoi, "sf") || inherits(aoi, "sfc"))) {
     rlang::abort(
@@ -364,6 +409,9 @@ get_stac_data <- function(aoi,
     gdal_config_options = gdal_config_options,
     ...
   )
+  if (!is.null(stats::na.action(download_results))) {
+    items$features[stats::na.action(download_results)] <- NULL
+  }
   # mask
   if (!is.null(mask_band)) {
     download_results <- rsi_apply_masks(
@@ -464,25 +512,8 @@ get_sentinel1_imagery <- function(aoi,
                                   output_filename = paste0(proceduralnames::make_english_names(1), ".tif"),
                                   composite_function = "median",
                                   limit = 999,
-                                  gdalwarp_options = c(
-                                    "-r", "bilinear",
-                                    "-multi",
-                                    "-overwrite",
-                                    "-co", "COMPRESS=DEFLATE",
-                                    "-co", "PREDICTOR=2",
-                                    "-co", "NUM_THREADS=ALL_CPUS"
-                                  ),
-                                  gdal_config_options = c(
-                                    VSI_CACHE = "TRUE",
-                                    GDAL_CACHEMAX = "30%",
-                                    VSI_CACHE_SIZE = "10000000",
-                                    GDAL_HTTP_MULTIPLEX = "YES",
-                                    GDAL_INGESTED_BYTES_AT_OPEN = "32000",
-                                    GDAL_DISABLE_READDIR_ON_OPEN = "EMPTY_DIR",
-                                    GDAL_HTTP_VERSION = "2",
-                                    GDAL_HTTP_MERGE_CONSECUTIVE_RANGES = "YES",
-                                    GDAL_NUM_THREADS = "ALL_CPUS"
-                                  )) {
+                                  gdalwarp_options = rsi_gdalwarp_options(),
+                                  gdal_config_options = rsi_gdal_config_options()) {
   args <- mget(names(formals()))
   args$`...` <- NULL
   args <- c(args, rlang::list2(...))
@@ -510,25 +541,8 @@ get_sentinel2_imagery <- function(aoi,
                                   output_filename = paste0(proceduralnames::make_english_names(1), ".tif"),
                                   composite_function = "median",
                                   limit = 999,
-                                  gdalwarp_options = c(
-                                    "-r", "bilinear",
-                                    "-multi",
-                                    "-overwrite",
-                                    "-co", "COMPRESS=DEFLATE",
-                                    "-co", "PREDICTOR=2",
-                                    "-co", "NUM_THREADS=ALL_CPUS"
-                                  ),
-                                  gdal_config_options = c(
-                                    VSI_CACHE = "TRUE",
-                                    GDAL_CACHEMAX = "30%",
-                                    VSI_CACHE_SIZE = "10000000",
-                                    GDAL_HTTP_MULTIPLEX = "YES",
-                                    GDAL_INGESTED_BYTES_AT_OPEN = "32000",
-                                    GDAL_DISABLE_READDIR_ON_OPEN = "EMPTY_DIR",
-                                    GDAL_HTTP_VERSION = "2",
-                                    GDAL_HTTP_MERGE_CONSECUTIVE_RANGES = "YES",
-                                    GDAL_NUM_THREADS = "ALL_CPUS"
-                                  )) {
+                                  gdalwarp_options = rsi_gdalwarp_options(),
+                                  gdal_config_options = rsi_gdal_config_options()) {
   args <- mget(names(formals()))
   args$`...` <- NULL
   args <- c(args, rlang::list2(...))
@@ -557,25 +571,8 @@ get_landsat_imagery <- function(aoi,
                                 output_filename = paste0(proceduralnames::make_english_names(1), ".tif"),
                                 composite_function = "median",
                                 limit = 999,
-                                gdalwarp_options = c(
-                                  "-r", "bilinear",
-                                  "-multi",
-                                  "-overwrite",
-                                  "-co", "COMPRESS=DEFLATE",
-                                  "-co", "PREDICTOR=2",
-                                  "-co", "NUM_THREADS=ALL_CPUS"
-                                ),
-                                gdal_config_options = c(
-                                  VSI_CACHE = "TRUE",
-                                  GDAL_CACHEMAX = "30%",
-                                  VSI_CACHE_SIZE = "10000000",
-                                  GDAL_HTTP_MULTIPLEX = "YES",
-                                  GDAL_INGESTED_BYTES_AT_OPEN = "32000",
-                                  GDAL_DISABLE_READDIR_ON_OPEN = "EMPTY_DIR",
-                                  GDAL_HTTP_VERSION = "2",
-                                  GDAL_HTTP_MERGE_CONSECUTIVE_RANGES = "YES",
-                                  GDAL_NUM_THREADS = "ALL_CPUS"
-                                )) {
+                                gdalwarp_options = rsi_gdalwarp_options(),
+                                gdal_config_options = rsi_gdal_config_options()) {
   args <- mget(names(formals()))
   args$`...` <- NULL
   args <- c(args, rlang::list2(...))
@@ -600,25 +597,8 @@ get_naip_imagery <- function(aoi,
                              output_filename = paste0(proceduralnames::make_english_names(1), ".tif"),
                              composite_function = "merge",
                              limit = 999,
-                             gdalwarp_options = c(
-                               "-r", "bilinear",
-                               "-multi",
-                               "-overwrite",
-                               "-co", "COMPRESS=DEFLATE",
-                               "-co", "PREDICTOR=2",
-                               "-co", "NUM_THREADS=ALL_CPUS"
-                             ),
-                             gdal_config_options = c(
-                               VSI_CACHE = "TRUE",
-                               GDAL_CACHEMAX = "30%",
-                               VSI_CACHE_SIZE = "10000000",
-                               GDAL_HTTP_MULTIPLEX = "YES",
-                               GDAL_INGESTED_BYTES_AT_OPEN = "32000",
-                               GDAL_DISABLE_READDIR_ON_OPEN = "EMPTY_DIR",
-                               GDAL_HTTP_VERSION = "2",
-                               GDAL_HTTP_MERGE_CONSECUTIVE_RANGES = "YES",
-                               GDAL_NUM_THREADS = "ALL_CPUS"
-                             )) {
+                             gdalwarp_options = rsi_gdalwarp_options(),
+                             gdal_config_options = rsi_gdal_config_options()) {
   args <- mget(names(formals()))
   args$`...` <- NULL
   args <- c(args, rlang::list2(...))
@@ -649,25 +629,8 @@ get_alos_palsar_imagery <- function(aoi,
                                     output_filename = paste0(proceduralnames::make_english_names(1), ".tif"),
                                     composite_function = "median",
                                     limit = 999,
-                                    gdalwarp_options = c(
-                                      "-r", "bilinear",
-                                      "-multi",
-                                      "-overwrite",
-                                      "-co", "COMPRESS=DEFLATE",
-                                      "-co", "PREDICTOR=2",
-                                      "-co", "NUM_THREADS=ALL_CPUS"
-                                    ),
-                                    gdal_config_options = c(
-                                      VSI_CACHE = "TRUE",
-                                      GDAL_CACHEMAX = "30%",
-                                      VSI_CACHE_SIZE = "10000000",
-                                      GDAL_HTTP_MULTIPLEX = "YES",
-                                      GDAL_INGESTED_BYTES_AT_OPEN = "32000",
-                                      GDAL_DISABLE_READDIR_ON_OPEN = "EMPTY_DIR",
-                                      GDAL_HTTP_VERSION = "2",
-                                      GDAL_HTTP_MERGE_CONSECUTIVE_RANGES = "YES",
-                                      GDAL_NUM_THREADS = "ALL_CPUS"
-                                    )) {
+                                    gdalwarp_options = rsi_gdalwarp_options(),
+                                    gdal_config_options = rsi_gdal_config_options()) {
   args <- mget(names(formals()))
   args$`...` <- NULL
   args <- c(args, rlang::list2(...))
@@ -695,25 +658,8 @@ get_dem <- function(aoi,
                     output_filename = paste0(proceduralnames::make_english_names(1), ".tif"),
                     composite_function = "max",
                     limit = 999,
-                    gdalwarp_options = c(
-                      "-r", "bilinear",
-                      "-multi",
-                      "-overwrite",
-                      "-co", "COMPRESS=DEFLATE",
-                      "-co", "PREDICTOR=2",
-                      "-co", "NUM_THREADS=ALL_CPUS"
-                    ),
-                    gdal_config_options = c(
-                      VSI_CACHE = "TRUE",
-                      GDAL_CACHEMAX = "30%",
-                      VSI_CACHE_SIZE = "10000000",
-                      GDAL_HTTP_MULTIPLEX = "YES",
-                      GDAL_INGESTED_BYTES_AT_OPEN = "32000",
-                      GDAL_DISABLE_READDIR_ON_OPEN = "EMPTY_DIR",
-                      GDAL_HTTP_VERSION = "2",
-                      GDAL_HTTP_MERGE_CONSECUTIVE_RANGES = "YES",
-                      GDAL_NUM_THREADS = "ALL_CPUS"
-                    )) {
+                    gdalwarp_options = rsi_gdalwarp_options(),
+                    gdal_config_options = rsi_gdal_config_options()) {
   args <- mget(names(formals()))
   args$`...` <- NULL
   args <- c(args, rlang::list2(...))
